/*
******************************************************************************
*  Name     : CopyUtils.h
*  Date     : 3.8.04
*  Description:
*     Static methods for fast bitmap copying. 
*	  Alpha modes implemented.
*     
*  Copyright: Susi Games, Germany
******************************************************************************
*/


#ifndef __CopyUtils_H__
#define __CopyUtils_H__

#include <e32base.h>
#include "FastBitmap.h"

/**
 * class CCopyUtils: Static methods to copy bitmaps.
 *
 * @author SusiGames.com
 */
class CCopyUtils : public CBase
{
public:

    ~CCopyUtils();


	/**
	 * BitBltMasked: Use this method if mask (2 alpha values:on/off) information is stored in bitmap data. (sus image)
	 * Clipping is implemented.
	 *
	 * @param aSrc The source EColor4K image. (load from sus file)
	 * @param aDst The destination bitmap in EColor4K mode.
	 * @param aPositionInDst The position in the destination bitmap.
	 * @param aFlipVertical If ETrue the copy will be flipped vertically.
	 * @return void 
	 */
	IMPORT_C static void BitBltMasked (CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInDst, TBool aFlipVertical = EFalse);


	/**
	 * BitBltMasked_4AlphaValues: Use this method for alpha based images.
	 * Only 4 alpha values supported.
	 *
	 * @param aSrc The source EColor4K image with alpha information. (load from sus file)
	 * @param aDst The destination bitmap in EColor4K mode.
	 * @param aPositionInDst The position in the destination bitmap.
	 * @param aFlipVertical If ETrue the copy will be flipped vertically.
	 * @return void 
	 */
	IMPORT_C static void BitBltMasked_4AlphaValues (CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInDst, TBool aFlipVertical = EFalse);


	/**
	 * BitBlt: Copies the complete bitmap aSrc to aDst at position aPositionInDst.
	 * Clipping is implemented.
	 * In EColor4K mode.
	 *
	 * @param aSrc The source EColor4K image.
	 * @param aDst The destination bitmap in EColor4K mode.
	 * @param aPositionInDst The position in the destination bitmap.
	 * @return void 
	 */
	IMPORT_C static void BitBlt_4K (CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInDst);



	/**
	 * BitBltCompleteDst: Copies a part of aSrc to aDst. Fills the complete aDst bitmap.
	 * Starts at positition aPositionInSrc in src bitmap.
	 *
	 * @param aSrc The source image.
	 * @param aDst The destination bitmap.
	 * @param aPositionInSrc The position in the source bitmap.
	 * @return void 
	 */
	IMPORT_C static void BitBltCompleteDst(CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInSrc);


	/**
	 * BitBltPartOfSrcToDst_4K: Copies the a part of aSrc to the complete bitmap aDst.
	 * The dimensions of the copy-part are given with the size of aDst.
	 * For EColor4K bitmaps.
	 *
	 * @param aSrc The source EColor4K image with alpha information.
	 * @param aDst The destination bitmap in EColor4K mode.
	 * @param aPositionInSrc The position from where to copy in the source image.
	 * @return void 
	 */
	IMPORT_C static void BitBltPartOfSrcToDst_4K (CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInSrc);



	/**
	 * BitBltMaskedAreaOfSrc: Use this method if mask information is stored in bitmap data. (all sus images)
	 * Only the given area of src bitmap is copied to destination.
	 *
	 * @param aSrc The source EColor4K image. (load from sus file)
	 * @param aDst The destination bitmap in EColor4K mode.
	 * @param aPositionInSrc The position in the source bitmap.
	 * @param aSizeInSrc size of area in source bitmap to copy.
	 * @param aPositionInDst The position in the destination bitmap.
	 * @param aFlipVertical If ETrue the copy will be flipped vertically.
	 * @return IMPORT_C void 
	 */
	IMPORT_C static void BitBltMaskedAreaOfSrc(CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInSrc, const TPoint &aSizeInSrc,const TPoint &aPositionInDst, TBool aFlipVertical /* = EFalse */);
	IMPORT_C static void BitBltMaskedAreaOfSrc_4AlphaValues(CFastBitmap *aSrc, CFastBitmap *aDst, const TPoint &aPositionInSrc, const TPoint &aSizeInSrc,const TPoint &aPositionInDst, TBool aFlipVertical /* = EFalse */);

private:

	// do not create an instance of this class.
    CCopyUtils();

};

#endif // __CopyUtils_H__

