/*
******************************************************************************
*  Name     : ImageLoader.h
*  Date     : 28.9.04
*  Description:
*     A Class for loading susImages.
*	  Imageloader works with reference counting, so an image 
*	  is never loaded twice. 
*     If an image is requested twice the imageloader will load the 
*	  image on the first request.
*	  On any further request the imageloader will return a pointer 
*	  to the already loaded image.
*	  So dont delete the Images somewhere. Just call ReleaseImage
*	  and the imageloader does this job for you.
*	  
*	  There should only be one imageloader for your application.
*	  You can achieve this in an easy and comfortable way by
*	  using static objects.
*     
*  Copyright: Susi Games, Germany
******************************************************************************
*/

#ifndef __IMAGELOADER_H__
#define __IMAGELOADER_H__

#include <e32base.h>
#include <eikenv.h>
#include <fbs.h>
#include "SusFile.h"


/**
 * class CImageLoader: A Class for loading susImages.
 *	  Imageloader works with reference counting, so an image 
 *	  is never loaded twice. 
 *    If an image is requested twice the imageloader will load the 
 *	  image on the first request.
 *	  On any further request the imageloader will return a pointer 
 *	  to the already loaded image.
 *	  So dont delete the Images somewhere. Just call ReleaseImage
 *	  and the imageloader does this job for you.
 *
 * @author SusiGames.com
 */
class CImageLoader : public CBase
{
public:

	/**
	 * RegisterSusFile: Use this method to inform the imageloader about a sus image file.
	 * After calling this method the header of the sus file is read and 
	 * the gfx-indexes are registered.
	 *
	 * @param aFilename The susImage file.
	 * @return TBool EFalse if filename not found or file corrupt.
	 */
	IMPORT_C TBool RegisterSusFile(const TDesC *aFilename);


	/**
	 * LoadSusImages: Loads sus images and uncompresses them. If the images are
	 * already loaded, the reference counting for that image increases and the already loaded image is stored
	 * in aStoreImages. Do not delete images: always release the images with ReleaseImage(aImage)
	 *
	 * @param aGFXIndex The gfx index. Defined on sus file creation.
	 * @param aStoreImages The loaded images are stored here.
	 * @param aImages How many images.
	 * @param aCheckInReferenceList To speed up the loading procedure:
	 *								If this loading is definitely the first access on the aGFXIndex images, 
	 *								you can set aCheckInReferenceList to EFalse, so this method
	 *								does not check the reference list for each image.
	 * @return TBool ETrue on success
	 */
	IMPORT_C TBool LoadSusImages(TUint32 aStartGFXIndex, RPointerArray<CFastBitmap> &aStoreImages, TInt aImages, TBool aCheckInReferenceList  = ETrue);


	/**
	 * LoadSusImage: Load one sus image.
	 *
	 * @param aGFXIndex The gfx index. Defined on sus file creation.
	 * @param aCheckInReferenceList To speed up the loading procedure:
	 *								If this loading is definitely the first access on the aGFXIndex image, 
	 *								you can set aCheckInReferenceList to EFalse, so this method
	 *								does not check the reference list for this image.
	 *
	 * @return  CFastBitmap* NULL if loading failed.
	 */
	IMPORT_C CFastBitmap* LoadSusImage(TUint32 aGFXIndex, TBool aCheckInReferenceList  = ETrue);


	/**
	 * ReleaseImage: Do not delete the requested images by yourself!!!
	 * Use this function if requester does not need the image any more.
	 * The requester must call this method, so refernce counting works well!
	 * This method just decreases the reference counting for the image.
	 * If it reaches zero the image will be deleted.
	 *
	 * @param aImageToRelease The image that is not needed any more.
	 * @return TBool ETrue on success.
	 */
	IMPORT_C TBool ReleaseImage(CFastBitmap* aImageToRelease);


	/**
	 * LoadingFinished: Call this method if application finished all sus-loading operations with this imageloader,
	 * to give free some memory. After calling LoadingFinished it is not possible to load sus images.
	 *
	 * @return void 
	 */
	IMPORT_C void LoadingFinished();


	/**
	 * IsImageLoaded: Checks if image represented by aGFXIndex is already loaded.
	 *
	 * @param aGFXIndex 
	 * @return TBool 
	 */
	TBool IsImageLoaded(TInt aGFXIndex);


	/**
	 * IncreaseReference: With this method you can manually increase the reference counting for an image.
	 *
	 * @param aImage 
	 * @return void 
	 */
	IMPORT_C void IncreaseReference(CFastBitmap* aImage);
	

    /**
     * NewL: To create the imageloader.
     *
     * @return CImageLoader* 
     */
    IMPORT_C static CImageLoader* NewL();


    /**
     * NewLC: To create the imageloader.
     *
     * @return CImageLoader* 
     */
    IMPORT_C static CImageLoader* NewLC();


	/**
	 * GetFilename: Returns pointer to filename associated with aGFXIndex
	 *
	 * @param aGFXIndex 
	 * @return TDesC* 
	 */
	IMPORT_C TDesC* GetFilename(TInt aGFXIndex);


	/**
	 * GetSusFile: Returns the associated sus file for the gfx index.
	 *
	 * @param aGFXIndex 
	 * @return CSusFile* 
	 */
	IMPORT_C CSusFile* GetSusFile(TInt aGFXIndex);


	/**
	 * GetApplicationPath: Writes the actual application path to aBuffer. (looks like this "e:\system\apps\yourapp\")
	 *
	 * @param aBuffer 
	 * @return void 
	 */
	IMPORT_C void GetApplicationPath( TDes * aBuffer );


    /**
     * ~CImageLoader: Destructor.
     *
     * @return  
     */
    ~CImageLoader();



private:


    CImageLoader();
    void ConstructL();


	/**
	 * LoadSusImage: Load sus images from stream. Header must be loaded.
	 *
	 * @param aInStream 
	 * @param aGFXIndex 
	 * @param aStoreImages 
	 * @param aImages 
	 * @param aCheckInReferenceList 
	 * @return TBool 
	 */
	TBool LoadSusImages(CSusFile *aSusFile, RPointerArray<CFastBitmap> &aStoreImages, TUint32 aGFXIndex, TInt aImages, TBool aCheckInReferenceList);


	/**
	 * LoadSusImage: Load sus image from stream. Header must be loaded.
	 *
	 * @param aSusFile 
	 * @param aGFXIndex 
	 * @param aCheckInReferenceList 
	 * @return CFastBitmap* 
	 */
	CFastBitmap* LoadSusImage(CSusFile *aSusFile, TUint32 aGFXIndex, TBool aCheckInReferenceList);
	

	/**
	 * DestroyImages: Delete all images in aStoreImages.
	 *
	 * @param aStoreImages 
	 * @return void 
	 */
	void DestroyImages(RPointerArray<CFastBitmap> &aStoreImages);


	/**
	 * FindImageInReferenceList Returns number of aImage in iImageReferenceList. If not in List -1 is returned.
	 *
	 * @param aImage The image to search
	 * @return TInt Position in List. -1 if not in list.
	 */
	TInt FindImageInReferenceList(CFastBitmap* aImage);
	TInt FindImageInReferenceList(TUint32 aGFXIndex);


	/**
	 * struct ImageReferences: A structure for storing the references to the loaded images (SusImg).
	 *
	 * @author SusiGames.com
	 */
	struct ImageReferences
	{
		CFastBitmap* Image;
		TInt referenceCount;
		CSusFile *SusFile;
		TDisplayMode ColorMode;
		TUint32 GFXIndex;
	};


	/**
	 * iImageReferenceList: Storing the Refences to loaded images (async loading / filenames).
	 */
	RPointerArray<ImageReferences> iImageReferenceList;


	/**
	 * iSusFiles: The loaded sus files.
	 */
	RPointerArray<CSusFile> iSusFiles;

};

#endif // __IMAGELOADER_H__

