/*
******************************************************************************
*  Name     : SusFile.h
*  Date     : 3.8.04
*  Description:
*     This Class represents a complete susImage file.
*	  Call LoadHeader() before and LoadPalettes() before calling LoadImage().
*     
*  Copyright: Susi Games, Germany
******************************************************************************
*/


#if !defined(__SUSFILE_H__)
#define __SUSFILE_H__

#include <e32base.h>
#include "fastbitmap.h"
#include "FastReadStream.h"


/**
 * class CSusFile: This Class represents a complete susImage file.
 *
 * @author SusiGames.com
 */
class CSusFile : public CBase  
{
public:

	/**
	 * CSusFile: Standard Constructor. Creates an empty class.
	 *
	 * @return  
	 */
	CSusFile();


	/**
	 * SetFilename: Set the filename of the sus file.
	 *
	 * @param aFilename 
	 * @return void 
	 */
	void SetFilename (const TDesC* aFilename);


	/**
	 * LoadHeader: Load header of sus file.
	 *
	 * @return TBool 
	 */
	TBool LoadHeader();


	/**
	 * LoadImage: If header is loaded and aGFXIndex is in the sus file,
	 * the bitmap represented by aGFXIndex is loaded.
	 *
	 * @param aGFXIndex The gfxIndex (set on creation of the sus file)
	 * @return CFastBitmap* 
	 */
	CFastBitmap* LoadImage(TInt aGFXIndex);


	/**
	 * IsGFXIndexInSusFile: To test if aGFXIndex is in the sus file.
	 * Call this method if header is loaded.
	 * After LoadingFinished() the header is unloaded->so we always get EFalse...
	 *
	 * @param aGFXIndex 
	 * @return TBool 
	 */
	TBool IsGFXIndexInSusFile(TUint32 aGFXIndex);



	/**
	 * GetFilename: To get the filename of the sus file.
	 *
	 * @return TDesC* 
	 */
	TDesC* GetFilename();


	/**
	 * LoadingFinished: Unload the header. Give free all used resources. 
	 * (header informations, bitmap informations, palettes)
	 *
	 * @return void 
	 */
	void LoadingFinished();


	/**
	 * LoadPalettes: Load all palettes needed for the images aStartIndex until aStartIndex+aCount.
	 *
	 * @param aStartIndex 
	 * @param aCount 
	 * @return TBool ETrue on success.
	 */
	TBool LoadPalettes(TInt aStartIndex, TInt aCount);

	virtual ~CSusFile();

private:

	//HeaderData:
	HBufC* iFilename;

	// file access
	CFastReadStream* iInStream;
	
	struct ImgSpec
	{
		TUint32 width,height;
		TUint16 index;
		TUint32 PositionInFile;
		TUint16 PaletteNr;
	};
	struct PaletteSpec
	{
		TUint16 index;
		TUint32 PositionInFile;
		TUint32 Entries;		// how many colors in palette
		TUint16 *Palette;		// the palette itself
	};
	struct SusImgHeader
	{
	  TBool valid;
	  TUint16 version;
	  TUint8 bpp;
	  TUint8 ChannelsColor;
	  TUint8 BitsColor;
	  TUint8 BitsAlpha;
	  TUint8 AlphaOrder;
	  TUint8 AlphaMode;
	  TUint8 RLE;
	  TUint32 width,height; // deprecated
	  TUint32 Images;		// number of images in file
	  RPointerArray<ImgSpec> ImageSpecifications; // specifications of images 
				 // !!! use ImageSpecifications.ResetAndDestroy() on deleting a header !!!!!
	  RPointerArray<PaletteSpec> PaletteSpecifications;
				// !!! use PaletteSpecifications.ResetAndDestroy() on deleting a header !!!!!
	  TUint32 BaseIndex;
	};

	// the header
	SusImgHeader* iHeader;


	/**
	 * DeleteHeader: Destroy all objects of the header.
	 *
	 * @return void 
	 */
	void DeleteHeader();


	/**
	 * LoadPaletteSusImgDataRLE: Load Image data from current position of aStream.
	 * Uses Palette to decode image.
	 *
	 * @param aStream 
	 * @param aHeader 
	 * @param aPalette 
	 * @param aPaletteEntries 
	 * @return static CFastBitmap* 
	 */
	CFastBitmap* LoadPaletteSusImgDataRLE(CFastReadStream *aStream, SusImgHeader *aHeader, TUint16* aPalette, TInt aPaletteEntries);


	/**
	 * ReadPalette: Read palette information from current position in stream.
	 *
	 * @param aStream 
	 * @param aHeader 
	 * @param aNr 
	 * @return TBool 
	 */
	TBool ReadPalette(CFastReadStream *aStream, SusImgHeader *aHeader,TInt aNr);
	TBool ReadPalettes(CFastReadStream *aStream, SusImgHeader *aHeader,TInt aFromIndex,TInt aImages);


	/**
	 * ReadSusImgPaletteSpecification_L: Reads all palette information from stream.
	 *
	 * @param aStream 
	 * @param aHeader 
	 * @return TBool 
	 */
	TBool ReadSusImgPaletteSpecification_L(CFastReadStream *aStream, SusImgHeader *aHeader);


	/**
	 * ReadImageSpecificationsL: Reads all image inforamtions from stream.
	 *
	 * @param aFile 
	 * @param aHeader 
	 * @return TInt 
	 */
	TInt ReadImageSpecificationsL(RFile &aFile, SusImgHeader &aHeader);


	/**
	 * ReadSusImgHeader_L: Uses Stream->GetFile file to read data.
	 * Does not update buffer of stream.
	 *
	 * @param aStream 
	 * @param aHeader 
	 * @return static TBool 
	 */
	TBool ReadSusImgHeader_L (CFastReadStream &aStream, SusImgHeader &aHeader);


	/**
	 * ReadHeaderL: To read the header.
	 *
	 * @param aFile 
	 * @param aHeader To store the header information.
	 * @return TInt The bytes read from File. -1 if something went wrong.
	 */
	TInt ReadHeaderL(RFile *aFile, SusImgHeader &aHeader);



};

#endif // !defined(__SUSFILE_H__)
